@@ -458,6 +458,36 @@ class RegressionTests(unittest.TestCase):
         self.assertEqual(first, second)
 
 
+    def test_sf_950057(self):
+        # Make sure that chain() and cycle() catch exceptions immediately
+        # rather than when shifting between input sources
+
+        def gen1():
+            hist.append(0)
+            yield 1
+            hist.append(1)
+            assert False
+            hist.append(2)
+
+        def gen2(x):
+            hist.append(3)
+            yield 2
+            hist.append(4)
+            if x:
+                raise StopIteration
+
+        hist = []
+        self.assertRaises(AssertionError, list, chain(gen1(), gen2(False)))
+        self.assertEqual(hist, [0,1])
+
+        hist = []
+        self.assertRaises(AssertionError, list, chain(gen1(), gen2(True)))
+        self.assertEqual(hist, [0,1])
+
+        hist = []
+        self.assertRaises(AssertionError, list, cycle(gen1()))
+        self.assertEqual(hist, [0,1])
+
 libreftest = """ Doctest for examples in the library reference: libitertools.tex
 
 
@@ -501,19 +531,19 @@ Samuele
 ...     "Returns the nth item"
 ...     return list(islice(iterable, n, n+1))
 
->>> def all(pred, seq):
+>>> def all(seq, pred=bool):
 ...     "Returns True if pred(x) is True for every element in the iterable"
 ...     return False not in imap(pred, seq)
 
->>> def some(pred, seq):
+>>> def any(seq, pred=bool):
 ...     "Returns True if pred(x) is True for at least one element in the iterable"
 ...     return True in imap(pred, seq)
 
->>> def no(pred, seq):
+>>> def no(seq, pred=bool):
 ...     "Returns True if pred(x) is False for every element in the iterable"
 ...     return True not in imap(pred, seq)
 
->>> def quantify(pred, seq):
+>>> def quantify(seq, pred=bool):
 ...     "Count how many times the predicate is True in the sequence"
 ...     return sum(imap(pred, seq))
 
@@ -554,25 +584,25 @@ perform as purported.
 >>> nth('abcde', 3)
 ['d']
 
->>> all(lambda x: x%2==0, [2, 4, 6, 8])
+>>> all([2, 4, 6, 8], lambda x: x%2==0)
 True
 
->>> all(lambda x: x%2==0, [2, 3, 6, 8])
+>>> all([2, 3, 6, 8], lambda x: x%2==0)
 False
 
->>> some(lambda x: x%2==0, [2, 4, 6, 8])
+>>> any([2, 4, 6, 8], lambda x: x%2==0)
 True
 
->>> some(lambda x: x%2==0, [1, 3, 5, 9])
+>>> any([1, 3, 5, 9], lambda x: x%2==0,)
 False
 
->>> no(lambda x: x%2==0, [1, 3, 5, 9])
+>>> no([1, 3, 5, 9], lambda x: x%2==0)
 True
 
->>> no(lambda x: x%2==0, [1, 2, 5, 9])
+>>> no([1, 2, 5, 9], lambda x: x%2==0)
 False
 
->>> quantify(lambda x: x%2==0, xrange(99))
+>>> quantify(xrange(99), lambda x: x%2==0)
 50
 
 >>> list(window('abc'))
